@@ -6,12 +6,9 @@
  * File state machine to handle a payload within an rpm package.
  */
 
-#include "cpio.h"
+#include <rpm/rpmfi.h>
 
-/*@-exportlocal@*/
-/*@unchecked@*/
 extern int _fsm_debug;
-/*@=exportlocal@*/
 
 /**
  */
@@ -37,7 +34,6 @@ typedef enum fileStage_e {
     FSM_PKGINSTALL	= _fd(7),
     FSM_PKGERASE	= _fd(8),
     FSM_PKGBUILD	= _fd(9),
-    FSM_PKGCOMMIT	= _fd(10),
     FSM_PKGUNDO		= _fd(11),
 
     FSM_CREATE	=  _fd(17),
@@ -54,18 +50,19 @@ typedef enum fileStage_e {
     FSM_RENAME	=  _fs(34),
     FSM_MKDIR	=  _fs(35),
     FSM_RMDIR	=  _fs(36),
-    FSM_CHOWN	=  _fs(37),
-    FSM_LCHOWN	=  _fs(38),
-    FSM_CHMOD	=  _fs(39),
-    FSM_UTIME	=  _fs(40),
-    FSM_SYMLINK	=  _fs(41),
-    FSM_LINK	=  _fs(42),
-    FSM_MKFIFO	=  _fs(43),
-    FSM_MKNOD	=  _fs(44),
-    FSM_LSTAT	=  _fs(45),
-    FSM_STAT	=  _fs(46),
-    FSM_READLINK=  _fs(47),
-    FSM_CHROOT	=  _fs(48),
+    FSM_LSETFCON=  _fs(39),
+    FSM_CHOWN	=  _fs(40),
+    FSM_LCHOWN	=  _fs(41),
+    FSM_CHMOD	=  _fs(42),
+    FSM_UTIME	=  _fs(43),
+    FSM_SYMLINK	=  _fs(44),
+    FSM_LINK	=  _fs(45),
+    FSM_MKFIFO	=  _fs(46),
+    FSM_MKNOD	=  _fs(47),
+    FSM_LSTAT	=  _fs(48),
+    FSM_STAT	=  _fs(49),
+    FSM_READLINK=  _fs(50),
+    FSM_SETCAP	=  _fs(52),
 
     FSM_NEXT	=  _fd(65),
     FSM_EAT	=  _fd(66),
@@ -90,80 +87,59 @@ typedef enum fileStage_e {
 #undef	_fd
 
 /** \ingroup payload
- * Keeps track of the set of all hard links to a file in an archive.
  */
-struct hardLink_s {
-/*@owned@*/ struct hardLink_s * next;
-/*@owned@*/ const char ** nsuffix;
-/*@owned@*/ int * filex;
-    struct stat sb;
-    int nlink;
-    int linksLeft;
-    int linkIndex;
-    int createdPath;
+enum cpioMapFlags_e {
+    CPIO_MAP_PATH	= (1 << 0),
+    CPIO_MAP_MODE	= (1 << 1),
+    CPIO_MAP_UID	= (1 << 2),
+    CPIO_MAP_GID	= (1 << 3),
+    CPIO_FOLLOW_SYMLINKS= (1 << 4), /*!< only for building. */
+    CPIO_MAP_ABSOLUTE	= (1 << 5),
+    CPIO_MAP_ADDDOT	= (1 << 6),
+    CPIO_ALL_HARDLINKS	= (1 << 7), /*!< fail if hardlinks are missing. */
+    CPIO_MAP_TYPE	= (1 << 8),  /*!< only for building. */
+    CPIO_SBIT_CHECK	= (1 << 9)
 };
+typedef rpmFlags cpioMapFlags;
 
-/** \ingroup payload
- * Iterator across package file info, forward on install, backward on erase.
- */
-struct fsmIterator_s {
-    rpmts ts;			/*!< transaction set. */
-    rpmfi fi;			/*!< transaction element file info. */
-    int reverse;		/*!< reversed traversal? */
-    int isave;			/*!< last returned iterator index. */
-    int i;			/*!< iterator index. */
-};
+typedef struct fsmIterator_s * FSMI_t;
+typedef struct fsm_s * FSM_t;
+
+typedef struct hardLink_s * hardLink_t;
 
 /** \ingroup payload
  * File name and stat information.
  */
 struct fsm_s {
-/*@owned@*/
-    const char * path;		/*!< Current file name. */
-/*@owned@*/
-    const char * opath;		/*!< Original file name. */
+    char * path;		/*!< Current file name. */
+    char * opath;		/*!< Original file name. */
     FD_t cfd;			/*!< Payload file handle. */
     FD_t rfd;			/*!<  read: File handle. */
-/*@dependent@*/
     char * rdbuf;		/*!<  read: Buffer. */
-/*@owned@*/
     char * rdb;			/*!<  read: Buffer allocated. */
     size_t rdsize;		/*!<  read: Buffer allocated size. */
     size_t rdlen;		/*!<  read: Number of bytes requested.*/
     size_t rdnb;		/*!<  read: Number of bytes returned. */
     FD_t wfd;			/*!< write: File handle. */
-/*@dependent@*/
     char * wrbuf;		/*!< write: Buffer. */
-/*@owned@*/
     char * wrb;			/*!< write: Buffer allocated. */
     size_t wrsize;		/*!< write: Buffer allocated size. */
     size_t wrlen;		/*!< write: Number of bytes requested.*/
     size_t wrnb;		/*!< write: Number of bytes returned. */
-/*@only@*/ /*@null@*/
     FSMI_t iter;		/*!< File iterator. */
     int ix;			/*!< Current file iterator index. */
-/*@only@*/
-    struct hardLink_s * links;	/*!< Pending hard linked file(s). */
-/*@only@*/
-    struct hardLink_s * li;	/*!< Current hard linked file(s). */
-/*@kept@*/ /*@null@*/
-    unsigned int * archiveSize;	/*!< Pointer to archive size. */
-/*@kept@*/ /*@null@*/
-    const char ** failedFile;	/*!< First file name that failed. */
-/*@shared@*/
+    hardLink_t links;		/*!< Pending hard linked file(s). */
+    hardLink_t li;		/*!< Current hard linked file(s). */
+    rpm_loff_t * archiveSize;	/*!< Pointer to archive size. */
+    rpm_loff_t archivePos;	/*!< Current archive position. */
+    rpm_loff_t cpioPos;
+    char ** failedFile;		/*!< First file name that failed. */
     const char * subdir;	/*!< Current file sub-directory. */
-/*@unused@*/
-    char subbuf[64];	/* XXX eliminate */
-/*@observer@*/
     const char * osuffix;	/*!< Old, preserved, file suffix. */
-/*@observer@*/
     const char * nsuffix;	/*!< New, created, file suffix. */
-/*@shared@*/
     const char * suffix;	/*!< Current file suffix. */
     char sufbuf[64];	/* XXX eliminate */
-/*@only@*/ /*@null@*/
     short * dnlx;		/*!< Last dirpath verified indexes. */
-/*@only@*/ /*@null@*/
     char * ldn;			/*!< Last dirpath verified. */
     int ldnlen;			/*!< Last dirpath current length. */
     int ldnalloc;		/*!< Last dirpath allocated length. */
@@ -171,23 +147,20 @@ struct fsm_s {
     int diskchecked;		/*!< Has stat(2) been performed? */
     int exists;			/*!< Does current file exist on disk? */
     int mkdirsdone;		/*!< Have "orphan" dirs been created? */
-    int astriplen;		/*!< Length of buildroot prefix. */
     int rc;			/*!< External file stage return code. */
-    int commit;			/*!< Commit synchronously? */
     cpioMapFlags mapFlags;	/*!< Bit(s) to control mapping. */
-/*@shared@*/
     const char * dirName;	/*!< File directory name. */
-/*@shared@*/
     const char * baseName;	/*!< File base name. */
-/*@shared@*/
-    const char * fmd5sum;	/*!< Hex MD5 sum (NULL disables). */
-/*@shared@*/
-    const char * md5sum;	/*!< Binary MD5 sum (NULL disables). */
+    const unsigned char * digest;	/*!< Binary digest (NULL disables). */
+    security_context_t fcontext;/*!< File security context (NULL disables). */
+    void *fcaps;		/*!< File capabilities */
+    pgpHashAlgo digestalgo;	/*!< File digest algorithm */
     
     unsigned fflags;		/*!< File flags. */
-    fileAction action;		/*!< File disposition. */
+    rpmFileAction action;	/*!< File disposition. */
     fileStage goal;		/*!< Package state machine goal. */
     fileStage stage;		/*!< External file stage. */
+    fileStage nstage;		/*!< Next file stage. */
     struct stat sb;		/*!< Current file stat(2) info. */
     struct stat osb;		/*!< Original file stat(2) info. */
 };
@@ -196,109 +169,56 @@ struct fsm_s {
 extern "C" {
 #endif
 
-/*@-exportlocal@*/
-/**
- * Return formatted string representation of file stages.
- * @param a		file stage
- * @return		formatted string
- */
-/*@observer@*/ const char *const fileStageString(fileStage a)	/*@*/;
-
-/**
- * Return formatted string representation of file disposition.
- * @param a		file dispostion
- * @return		formatted string
- */
-/*@observer@*/ const char *const fileActionString(fileAction a)	/*@*/;
-/*@=exportlocal@*/
-
 /**
  * Create file state machine instance.
- * @return		file state machine data
+ * @param mapflags	CPIO map flags to use
+ * @return		file state machine
  */
-/*@only@*/ FSM_t newFSM(void)
-	/*@*/;
+RPM_GNUC_INTERNAL
+FSM_t newFSM(cpioMapFlags mapflags);
 
 /**
  * Destroy file state machine instance.
- * @param fsm		file state machine data
+ * @param fsm		file state machine
  * @return		always NULL
  */
-/*@null@*/ FSM_t freeFSM(/*@only@*/ /*@null@*/ FSM_t fsm)
-	/*@globals fileSystem @*/
-	/*@modifies fsm, fileSystem @*/;
+RPM_GNUC_INTERNAL
+FSM_t freeFSM(FSM_t fsm);
 
 /**
  * Load external data into file state machine.
- * @param fsm		file state machine data
+ * @param fsm		file state machine
  * @param goal
  * @param ts		transaction set
  * @param fi		transaction element file info
  * @param cfd
  * @retval archiveSize	pointer to archive size
- * @retval failedFile	pointer to first file name that failed.
+ * @retval failedFile	pointer to first file name that failed (malloced)
  * @return		0 on success
  */
 int fsmSetup(FSM_t fsm, fileStage goal,
-		const rpmts ts,
-		const rpmfi fi,
+		rpmts ts,
+		rpmte te,
+		rpmfi fi,
 		FD_t cfd,
-		/*@out@*/ unsigned int * archiveSize,
-		/*@out@*/ const char ** failedFile)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fsm, ts, fi, *archiveSize, *failedFile,
-		fileSystem, internalState @*/;
+		rpm_loff_t * archiveSize,
+		char ** failedFile);
 
 /**
  * Clean file state machine.
- * @param fsm		file state machine data
+ * @param fsm		file state machine
  * @return		0 on success
  */
-int fsmTeardown(FSM_t fsm)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fsm, fileSystem, internalState @*/;
-
-/*@-exportlocal@*/
-/**
- * Retrieve transaction set from file state machine iterator.
- * @param fsm		file state machine data
- * @return		transaction set
- */
-rpmts fsmGetTs(const FSM_t fsm)
-	/*@*/;
-
-/**
- * Retrieve transaction element file info from file state machine iterator.
- * @param fsm		file state machine data
- * @return		transaction element file info
- */
-rpmfi fsmGetFi(/*@partial@*/ const FSM_t fsm)
-	/*@*/;
-
-/**
- * Map next file path and action.
- * @param fsm		file state machine data
- */
-int fsmMapPath(FSM_t fsm)
-	/*@modifies fsm @*/;
-
-/**
- * Map file stat(2) info.
- * @param fsm		file state machine data
- */
-int fsmMapAttrs(FSM_t fsm)
-	/*@modifies fsm @*/;
-/*@=exportlocal@*/
+int fsmTeardown(FSM_t fsm);
 
 /**
  * File state machine driver.
- * @param fsm		file state machine data
- * @param stage		next stage
+ * @param fsm		file state machine
+ * @param nstage		next stage
  * @return		0 on success
  */
-int fsmStage(/*@partial@*/ FSM_t fsm, fileStage stage)
-	/*@globals errno, fileSystem, internalState @*/
-	/*@modifies fsm, errno, fileSystem, internalState @*/;
+RPM_GNUC_INTERNAL
+int fsmNext(FSM_t fsm, fileStage nstage);
 
 #ifdef __cplusplus
 }
