@@ -4,8 +4,17 @@
  */
 
 #include "system.h"
+const char *__progname;
 
-#include <rpmcli.h>
+#if HAVE_MCHECK_H
+#include <mcheck.h>
+#endif
+
+#include <rpm/rpmcli.h>
+#include <rpm/rpmlib.h>		/* rpmEVR, rpmReadConfigFiles etc */
+#include <rpm/rpmlog.h>
+#include <rpm/rpmstring.h>
+#include <rpm/rpmfileutil.h>
 
 #include "debug.h"
 
@@ -13,94 +22,39 @@
 #define POPT_SHOWRC		-998
 #define POPT_QUERYTAGS		-997
 #define POPT_PREDEFINE		-996
-#ifdef  NOTYET
-#define POPT_RCFILE		-995
-#endif
+#define POPT_DBPATH		-995
 
-/*@unchecked@*/
 static int _debug = 0;
 
-/*@-exportheadervar@*/
-/*@unchecked@*/
 extern int _rpmds_nopromote;
 
-/*@unchecked@*/
-extern int _fps_debug;
-
-/*@unchecked@*/
 extern int _fsm_debug;
 
-/*@unchecked@*/
-extern int _hdr_debug;
-
-/*@unchecked@*/
 extern int _print_pkts;
 
-/*@unchecked@*/
 extern int _psm_debug;
 
-/*@unchecked@*/
-extern int _rpmal_debug;
-
-/*@unchecked@*/
-extern int _rpmdb_debug;
-
-/*@unchecked@*/
-extern int _rpmds_debug;
-
 /* XXX avoid -lrpmbuild linkage. */
-/*@unchecked@*/
        int _rpmfc_debug;
 
-/*@unchecked@*/
-extern int _rpmfi_debug;
-
-/*@unchecked@*/
-extern int _rpmps_debug;
-
-/*@unchecked@*/
-extern int _rpmte_debug;
-
-/*@unchecked@*/
-extern int _rpmts_debug;
-
-/*@unchecked@*/
 extern int _rpmts_stats;
 
-/*@unchecked@*/
-extern int noLibio;
-/*@=exportheadervar@*/
-
-/*@unchecked@*/
 const char * rpmcliPipeOutput = NULL;
 
-/*@unchecked@*/
 const char * rpmcliRcfile = NULL;
 
-/*@unchecked@*/
 const char * rpmcliRootDir = "/";
 
-/*@unchecked@*/
 rpmQueryFlags rpmcliQueryFlags;
 
-/*@-exportheadervar@*/
-/*@unchecked@*/
-extern int _ftp_debug;
-/*@unchecked@*/
-extern int noLibio;
-/*@unchecked@*/
 extern int _rpmio_debug;
-/*@=exportheadervar@*/
 
-/*@unchecked@*/
 static int rpmcliInitialized = -1;
 
 /**
  * Display rpm version.
  */
 static void printVersion(FILE * fp)
-	/*@globals rpmEVR, fileSystem @*/
-	/*@modifies *fp, fileSystem @*/
 {
     fprintf(fp, _("RPM version %s\n"), rpmEVR);
 }
@@ -109,12 +63,7 @@ static void printVersion(FILE * fp)
  * Make sure that config files have been read.
  * @warning Options like --rcfile and --verbose must precede callers option.
  */
-/*@mayexit@*/
 void rpmcliConfigured(void)
-	/*@globals rpmcliInitialized, rpmCLIMacroContext, rpmGlobalMacroContext,
-		fileSystem, internalState @*/
-	/*@modifies rpmcliInitialized, rpmCLIMacroContext, rpmGlobalMacroContext,
-		fileSystem, internalState @*/
 {
 
     if (rpmcliInitialized < 0)
@@ -125,23 +74,17 @@ void rpmcliConfigured(void)
 
 /**
  */
-/*@-bounds@*/
-static void rpmcliAllArgCallback( /*@unused@*/ poptContext con,
-                /*@unused@*/ enum poptCallbackReason reason,
+static void rpmcliAllArgCallback( poptContext con,
+                enum poptCallbackReason reason,
                 const struct poptOption * opt, const char * arg,
-                /*@unused@*/ const void * data)
-	/*@globals rpmcliQueryFlags, rpmCLIMacroContext, rpmGlobalMacroContext,
-		fileSystem, internalState @*/
-	/*@modifies rpmcliQueryFlags, rpmCLIMacroContext, rpmGlobalMacroContext,
-		fileSystem, internalState @*/
+                const void * data)
 {
 
     /* XXX avoid accidental collisions with POPT_BIT_SET for flags */
-    /*@-branchstate@*/
     if (opt->arg == NULL)
     switch (opt->val) {
     case 'q':
-	rpmSetVerbosity(RPMMESS_QUIET);
+	rpmSetVerbosity(RPMLOG_WARNING);
 	break;
     case 'v':
 	rpmIncreaseVerbosity();
@@ -150,39 +93,48 @@ static void rpmcliAllArgCallback( /*@unused@*/ poptContext con,
 	(void) rpmDefineMacro(NULL, arg, RMIL_CMDLINE);
 	break;
     case 'D':
+    {   char *s, *t;
+	/* XXX Convert '-' in macro name to underscore, skip leading %. */
+	s = t = xstrdup(arg);
+	while (*t && !risspace(*t)) {
+	    if (*t == '-') *t = '_';
+	    t++;
+	}
+	t = s;
+	if (*t == '%') t++;
 	/* XXX Predefine macro if not initialized yet. */
 	if (rpmcliInitialized < 0)
-	    (void) rpmDefineMacro(NULL, arg, RMIL_CMDLINE);
+	    (void) rpmDefineMacro(NULL, t, RMIL_CMDLINE);
 	rpmcliConfigured();
-/*@-type@*/
-	(void) rpmDefineMacro(NULL, arg, RMIL_CMDLINE);
-	(void) rpmDefineMacro(rpmCLIMacroContext, arg, RMIL_CMDLINE);
-/*@=type@*/
+	(void) rpmDefineMacro(NULL, t, RMIL_CMDLINE);
+	(void) rpmDefineMacro(rpmCLIMacroContext, t, RMIL_CMDLINE);
+	s = _free(s);
 	break;
+    }
     case 'E':
 	rpmcliConfigured();
-	{   const char *val = rpmExpand(arg, NULL);
+	{   char *val = rpmExpand(arg, NULL);
 	    fprintf(stdout, "%s\n", val);
 	    val = _free(val);
 	}
 	break;
+    case POPT_DBPATH:
+	rpmcliConfigured();
+	addMacro(NULL, "_dbpath", NULL, arg, RMIL_CMDLINE);
+	break;
     case POPT_SHOWVERSION:
 	printVersion(stdout);
 	exit(EXIT_SUCCESS);
-	/*@notreached@*/ break;
+	break;
     case POPT_SHOWRC:
 	rpmcliConfigured();
 	(void) rpmShowRC(stdout);
 	exit(EXIT_SUCCESS);
-	/*@notreached@*/ break;
+	break;
     case POPT_QUERYTAGS:
 	rpmDisplayQueryTags(stdout);
 	exit(EXIT_SUCCESS);
-	/*@notreached@*/ break;
-#if defined(POPT_RCFILE)
-    case POPT_RCFILE:		/* XXX FIXME: noop for now */
 	break;
-#endif
     case RPMCLI_POPT_NODIGEST:
 	rpmcliQueryFlags |= VERIFY_DIGEST;
 	break;
@@ -195,16 +147,12 @@ static void rpmcliAllArgCallback( /*@unused@*/ poptContext con,
 	rpmcliQueryFlags |= VERIFY_HDRCHK;
 	break;
     }
-    /*@=branchstate@*/
 }
 
-/*@-bitwisesigned -compmempass @*/
-/*@unchecked@*/
 struct poptOption rpmcliAllPoptTable[] = {
-/*@-type@*/ /* FIX: cast? */
+/* FIX: cast? */
  { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE,
         rpmcliAllArgCallback, 0, NULL, NULL },
-/*@=type@*/
 
  { "debug", 'd', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_debug, -1,
         NULL, NULL },
@@ -226,28 +174,21 @@ struct poptOption rpmcliAllPoptTable[] = {
         N_("don't verify package digest(s)"), NULL },
  { "nohdrchk", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, RPMCLI_POPT_NOHDRCHK,
         N_("don't verify database header(s) when retrieved"), NULL },
-#if HAVE_LIBIO_H && defined(_G_IO_IO_FILE_VERSION)
- { "nolibio", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &noLibio, 1,
-	N_("disable use of libio(3) API"), NULL},
-#endif
  { "nosignature", '\0', 0, 0, RPMCLI_POPT_NOSIGNATURE,
         N_("don't verify package signature(s)"), NULL },
 
  { "pipe", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, &rpmcliPipeOutput, 0,
 	N_("send stdout to CMD"),
 	N_("CMD") },
-#if !defined(POPT_RCFILE)
  { "rcfile", '\0', POPT_ARG_STRING, &rpmcliRcfile, 0,
 	N_("read <FILE:...> instead of default file(s)"),
 	N_("<FILE:...>") },
-#else
- { "rcfile", '\0', 0, NULL, POPT_RCFILE,	
-	N_("read <FILE:...> instead of default file(s)"),
-	N_("<FILE:...>") },
-#endif
  { "root", 'r', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &rpmcliRootDir, 0,
 	N_("use ROOT as top level directory"),
 	N_("ROOT") },
+ { "dbpath", '\0', POPT_ARG_STRING, 0, POPT_DBPATH,
+	N_("use database in DIRECTORY"),
+	N_("DIRECTORY") },
 
  { "querytags", '\0', 0, 0, POPT_QUERYTAGS,
         N_("display known query tags"), NULL },
@@ -260,85 +201,50 @@ struct poptOption rpmcliAllPoptTable[] = {
  { "version", '\0', 0, NULL, POPT_SHOWVERSION,
 	N_("print the version of rpm being used"), NULL },
 
-#if HAVE_LIBIO_H && defined(_G_IO_IO_FILE_VERSION)
- { "nolibio", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &noLibio, 1,
-       N_("disable use of libio(3) API"), NULL},
-#endif
-
  { "promoteepoch", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmds_nopromote, 0,
 	NULL, NULL},
 
- { "fpsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_fps_debug, -1,
-	NULL, NULL},
  { "fsmdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_fsm_debug, -1,
 	N_("debug payload file state machine"), NULL},
- { "ftpdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_ftp_debug, -1,
-	N_("debug protocol data stream"), NULL},
- { "hdrdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_hdr_debug, -1,
-	NULL, NULL},
-#ifdef	DYING
- { "poptdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_popt_debug, -1,
-	N_("debug option/argument processing"), NULL},
-#endif
  { "prtpkts", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_print_pkts, -1,
 	NULL, NULL},
- { "psmdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_psm_debug, -1,
-	N_("debug package state machine"), NULL},
- { "rpmaldebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmal_debug, -1,
-	NULL, NULL},
- { "rpmdbdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmdb_debug, -1,
-	NULL, NULL},
- { "rpmdsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmds_debug, -1,
-	NULL, NULL},
  { "rpmfcdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmfc_debug, -1,
 	NULL, NULL},
- { "rpmfidebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmfi_debug, -1,
-	NULL, NULL},
  { "rpmiodebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmio_debug, -1,
 	N_("debug rpmio I/O"), NULL},
- { "rpmpsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmps_debug, -1,
-	NULL, NULL},
- { "rpmtedebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmte_debug, -1,
-	NULL, NULL},
- { "rpmtsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmts_debug, -1,
-	NULL, NULL},
  { "stats", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmts_stats, -1,
 	NULL, NULL},
- { "urldebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_url_debug, -1,
-	N_("debug URL cache handling"), NULL},
 
    POPT_TABLEEND
 };
-/*@=bitwisesigned =compmempass @*/
 
 poptContext
 rpmcliFini(poptContext optCon)
 {
-    optCon = poptFreeContext(optCon);
+    poptFreeContext(optCon);
+    rpmFreeMacros(NULL);
+    rpmFreeMacros(rpmCLIMacroContext);
+    rpmFreeRpmrc();
+    rpmlogClose();
 
 #if HAVE_MCHECK_H && HAVE_MTRACE
-    /*@-noeffect@*/
     muntrace();   /* Trace malloc only if MALLOC_TRACE=mtrace-output-file. */
-    /*@=noeffect@*/
 #endif
 
     return NULL;
 }
 
-/*@-globstate@*/
 poptContext
 rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
 {
     const char * optArg;
     poptContext optCon;
     int rc;
+    const char *ctx, *execPath;
 
 #if HAVE_MCHECK_H && HAVE_MTRACE
-    /*@-noeffect@*/
     mtrace();   /* Trace malloc only if MALLOC_TRACE=mtrace-output-file. */
-    /*@=noeffect@*/
 #endif
-/*@-globs -mods@*/
     setprogname(argv[0]);       /* Retrofit glibc __progname */
 
     /* XXX glibc churn sanity */
@@ -346,15 +252,15 @@ rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
 	if ((__progname = strrchr(argv[0], '/')) != NULL) __progname++;
 	else __progname = argv[0];
     }
-/*@=globs =mods@*/
 
-#if defined(ENABLE_NLS) && !defined(__LCLINT__)
+#if defined(ENABLE_NLS)
     (void) setlocale(LC_ALL, "" );
+
     (void) bindtextdomain(PACKAGE, LOCALEDIR);
     (void) textdomain(PACKAGE);
 #endif
 
-    rpmSetVerbosity(RPMMESS_NORMAL);
+    rpmSetVerbosity(RPMLOG_NOTICE);
 
     if (optionsTable == NULL) {
 	/* Read rpm configuration (if not already read). */
@@ -362,34 +268,38 @@ rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
 	return NULL;
     }
 
-/*@-nullpass -temptrans@*/
-    optCon = poptGetContext(__progname, argc, (const char **)argv, optionsTable, 0);
-/*@=nullpass =temptrans@*/
-    (void) poptReadConfigFile(optCon, LIBRPMALIAS_FILENAME);
+    /* XXX hack to get popt working from build tree wrt lt-foo names */
+    ctx = rstreqn(__progname, "lt-", 3) ? __progname + 3 : __progname;
+
+    optCon = poptGetContext(ctx, argc, (const char **)argv, optionsTable, 0);
+    {
+	char *poptfile = rpmGenPath(rpmConfigDir(), LIBRPMALIAS_FILENAME, NULL);
+	(void) poptReadConfigFile(optCon, poptfile);
+	free(poptfile);
+    }
     (void) poptReadDefaultConfig(optCon, 1);
-    poptSetExecPath(optCon, RPMCONFIGDIR, 1);
+
+    if ((execPath = getenv("RPM_POPTEXEC_PATH")) == NULL)
+	execPath = LIBRPMALIAS_EXECPATH;
+    poptSetExecPath(optCon, execPath, 1);
 
     /* Process all options, whine if unknown. */
     while ((rc = poptGetNextOpt(optCon)) > 0) {
 	optArg = poptGetOptArg(optCon);
 	switch (rc) {
 	default:
-/*@-nullpass@*/
 	    fprintf(stderr, _("%s: option table misconfigured (%d)\n"),
 		__progname, rc);
-/*@=nullpass@*/
 	    exit(EXIT_FAILURE);
 
-	    /*@notreached@*/ /*@switchbreak@*/ break;
+	    break;
         }
     }
 
     if (rc < -1) {
-/*@-nullpass@*/
 	fprintf(stderr, "%s: %s: %s\n", __progname,
 		poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
 		poptStrerror(rc));
-/*@=nullpass@*/
 	exit(EXIT_FAILURE);
     }
 
@@ -403,4 +313,3 @@ rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
 
     return optCon;
 }
-/*@=globstate@*/
