@@ -136,7 +136,7 @@ static PyObject * hdrKeyList(hdrObject * s)
 {
     PyObject * keys = PyList_New(0);
     HeaderIterator hi = headerInitIterator(s->h);
-    rpmTag tag;
+    rpmTagVal tag;
 
     while ((tag = headerNextTag(hi)) != RPMTAG_NOT_FOUND) {
 	PyObject *to = PyInt_FromLong(tag);
@@ -165,7 +165,7 @@ static PyObject * hdrUnload(hdrObject * s, PyObject * args, PyObject *keywords)
 	h = headerCopy(s->h);	/* XXX strip region tags, etc */
 	headerFree(s->h);
     }
-    len = headerSizeof(h, 0);
+    len = headerSizeof(h, HEADER_MAGIC_NO);
     buf = headerUnload(h);
     h = headerFree(h);
 
@@ -248,7 +248,7 @@ static PyObject *hdrIsSource(hdrObject *s)
 
 static int hdrContains(hdrObject *s, PyObject *pytag)
 {
-    rpmTag tag;
+    rpmTagVal tag;
     if (!tagNumFromPyObject(pytag, &tag)) return -1;
 
     return headerIsEntry(s->h, tag);
@@ -257,7 +257,7 @@ static int hdrContains(hdrObject *s, PyObject *pytag)
 static PyObject *hdrConvert(hdrObject *self, PyObject *args, PyObject *kwds)
 {
     char *kwlist[] = {"op", NULL};
-    headerConvOps op = -1;
+    int op = -1;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &op)) {
         return NULL;
@@ -268,7 +268,7 @@ static PyObject *hdrConvert(hdrObject *self, PyObject *args, PyObject *kwds)
 static PyObject * hdrWrite(hdrObject *s, PyObject *args, PyObject *kwds)
 {
     char *kwlist[] = { "file", "magic", NULL };
-    int magic = 1;
+    int magic = HEADER_MAGIC_YES;
     rpmfdObject *fdo = NULL;
     int rc;
 
@@ -301,7 +301,7 @@ static PyObject * hdr_fiFromHeader(PyObject * s, PyObject * args, PyObject * kwd
 /* Backwards compatibility. Flags argument is just a dummy and discarded. */
 static PyObject * hdr_dsFromHeader(PyObject * s, PyObject * args, PyObject * kwds)
 {
-    rpmTag tag = RPMTAG_REQUIRENAME;
+    rpmTagVal tag = RPMTAG_REQUIRENAME;
     rpmsenseFlags flags = 0;
     char * kwlist[] = {"to", "flags", NULL};
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&i:dsFromHeader", kwlist,
@@ -401,7 +401,7 @@ static void hdr_dealloc(hdrObject * s)
 static PyObject * hdr_iternext(hdrObject *s)
 {
     PyObject *res = NULL;
-    rpmTag tag;
+    rpmTagVal tag;
 
     if (s->hi == NULL) s->hi = headerInitIterator(s->h);
 
@@ -428,9 +428,9 @@ int utf8FromPyObject(PyObject *item, PyObject **str)
     return 1;
 }
 
-int tagNumFromPyObject (PyObject *item, rpmTag *tagp)
+int tagNumFromPyObject (PyObject *item, rpmTagVal *tagp)
 {
-    rpmTag tag = RPMTAG_NOT_FOUND;
+    rpmTagVal tag = RPMTAG_NOT_FOUND;
     PyObject *str = NULL;
 
     if (PyInt_Check(item)) {
@@ -452,7 +452,7 @@ int tagNumFromPyObject (PyObject *item, rpmTag *tagp)
     return 1;
 }
 
-static PyObject * hdrGetTag(Header h, rpmTag tag)
+static PyObject * hdrGetTag(Header h, rpmTagVal tag)
 {
     PyObject *res = NULL;
     struct rpmtd_s td;
@@ -464,11 +464,11 @@ static PyObject * hdrGetTag(Header h, rpmTag tag)
     return res;
 }
 
-static int validItem(rpmTagClass class, PyObject *item)
+static int validItem(rpmTagClass tclass, PyObject *item)
 {
     int rc;
 
-    switch (class) {
+    switch (tclass) {
     case RPM_NUMERIC_CLASS:
 	rc = (PyLong_Check(item) || PyInt_Check(item));
 	break;
@@ -485,20 +485,19 @@ static int validItem(rpmTagClass class, PyObject *item)
     return rc;
 }
 
-static int validData(rpmTag tag, rpmTagType type, PyObject *value)
+static int validData(rpmTagVal tag, rpmTagType type, rpmTagReturnType retype, PyObject *value)
 {
-    rpmTagClass class = rpmTagGetClass(tag);
-    rpmTagReturnType retype = (type & RPM_MASK_RETURN_TYPE);
+    rpmTagClass tclass = rpmTagGetClass(tag);
     int valid = 1;
     
     if (retype == RPM_SCALAR_RETURN_TYPE) {
-	valid = validItem(class, value);
+	valid = validItem(tclass, value);
     } else if (retype == RPM_ARRAY_RETURN_TYPE && PyList_Check(value)) {
 	/* python lists can contain arbitrary objects, validate each item */
 	Py_ssize_t len = PyList_Size(value);
 	for (Py_ssize_t i = 0; i < len; i++) {
 	    PyObject *item = PyList_GetItem(value, i);
-	    if (!validItem(class, item)) {
+	    if (!validItem(tclass, item)) {
 		valid = 0;
 		break;
 	    }
@@ -509,11 +508,11 @@ static int validData(rpmTag tag, rpmTagType type, PyObject *value)
     return valid;
 }
 
-static int hdrAppendItem(Header h, rpmTag tag, rpmTagType type, PyObject *item)
+static int hdrAppendItem(Header h, rpmTagVal tag, rpmTagType type, PyObject *item)
 {
     int rc = 0;
 
-    switch ((type & RPM_MASK_TYPE)) {
+    switch (type) {
     case RPM_I18NSTRING_TYPE: /* XXX this needs to be handled separately */
     case RPM_STRING_TYPE:
     case RPM_STRING_ARRAY_TYPE: {
@@ -550,10 +549,10 @@ static int hdrAppendItem(Header h, rpmTag tag, rpmTagType type, PyObject *item)
     return rc;
 }
 
-static int hdrPutTag(Header h, rpmTag tag, PyObject *value)
+static int hdrPutTag(Header h, rpmTagVal tag, PyObject *value)
 {
-    rpmTagType type = rpmTagGetType(tag);
-    rpmTagReturnType retype = (type & RPM_MASK_RETURN_TYPE);
+    rpmTagType type = rpmTagGetTagType(tag);
+    rpmTagReturnType retype = rpmTagGetReturnType(tag);
     int rc = 0;
 
     /* XXX this isn't really right (i18n strings etc) but for now ... */
@@ -563,7 +562,7 @@ static int hdrPutTag(Header h, rpmTag tag, PyObject *value)
     }
 
     /* validate all data before trying to insert */
-    if (!validData(tag, type, value)) { 
+    if (!validData(tag, type, retype, value)) { 
 	PyErr_SetString(PyExc_TypeError, "invalid type for tag");
 	return 0;
     }
@@ -585,7 +584,7 @@ static int hdrPutTag(Header h, rpmTag tag, PyObject *value)
 
 static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
 {
-    rpmTag tag;
+    rpmTagVal tag;
 
     if (!tagNumFromPyObject(item, &tag)) return NULL;
     return hdrGetTag(s->h, tag);
@@ -593,7 +592,7 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
 
 static int hdr_ass_subscript(hdrObject *s, PyObject *key, PyObject *value)
 {
-    rpmTag tag;
+    rpmTagVal tag;
     if (!tagNumFromPyObject(key, &tag)) return -1;
 
     if (value == NULL) {
@@ -609,7 +608,7 @@ static PyObject * hdr_getattro(hdrObject * s, PyObject * n)
 {
     PyObject *res = PyObject_GenericGetAttr((PyObject *) s, n);
     if (res == NULL) {
-	rpmTag tag;
+	rpmTagVal tag;
 	if (tagNumFromPyObject(n, &tag)) {
 	    PyErr_Clear();
 	    res = hdrGetTag(s->h, tag);
@@ -716,7 +715,7 @@ static int rpmMergeHeaders(PyObject * list, FD_t fd, int matchTag)
 {
     Header h;
     HeaderIterator hi;
-    rpmTag newMatch, oldMatch;
+    rpmTagVal newMatch, oldMatch;
     hdrObject * hdr;
     rpm_count_t count = 0;
     int rc = 1; /* assume failure */
