@@ -1,20 +1,13 @@
-/** \ingroup py_c
- * \file python/rpmte-py.c
- */
-
-#include "system.h"
-
-#include <rpmlib.h>
+#include "rpmsystem-py.h"
 
 #include "header-py.h"	/* XXX tagNumFromPyObject */
 #include "rpmds-py.h"
 #include "rpmfi-py.h"
 #include "rpmte-py.h"
+#include "rpmps-py.h"
 
 #include "debug.h"
 
-/*@access rpmte @*/
-/*@access fnpyKey @*/
 
 /** \ingroup python
  * \name Class: Rpmte
@@ -37,11 +30,8 @@
  * - te.NEVR()	Return package name-version-release.
  * - te.Color() Return package color bits.
  * - te.PkgFileSize() Return no. of bytes in package file (approx).
- * - te.Depth()	Return the level in the dependency tree (after ordering).
- * - te.Npreds() Return the number of package prerequisites (after ordering).
- * - te.Degree() Return the parent's degree + 1.
  * - te.Parent() Return the parent element index.
- * - te.Tree()	Return the root dependency tree index.
+ * - te.Problems() Return problems associated with this element.
  * - te.AddedKey() Return the added package index (TR_ADDED).
  * - te.DependsOnKey() Return the package index for the added package (TR_REMOVED).
  * - te.DBOffset() Return the Packages database instance number (TR_REMOVED)
@@ -52,170 +42,108 @@
  * @param tag	'Basenames'
  */
 
-/*@null@*/
-static PyObject *
-rpmte_Debug(/*@unused@*/ rpmteObject * s, PyObject * args, PyObject * kwds)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies _Py_NoneStruct @*/
-{
-    char * kwlist[] = {"debugLevel", NULL};
+struct rpmteObject_s {
+    PyObject_HEAD
+    PyObject *md_dict;		/*!< to look like PyModuleObject */
+    rpmte	te;
+};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &_rpmte_debug))
-	return NULL;
-
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-
-/*@null@*/
 static PyObject *
 rpmte_TEType(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("i", rpmteType(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_N(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("s", rpmteN(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_E(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("s", rpmteE(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_V(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("s", rpmteV(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_R(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("s", rpmteR(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_A(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("s", rpmteA(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_O(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("s", rpmteO(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_NEVR(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("s", rpmteNEVR(s->te));
 }
 
-/*@null@*/
+static PyObject *
+rpmte_NEVRA(rpmteObject * s)
+{
+    return Py_BuildValue("s", rpmteNEVRA(s->te));
+}
+
 static PyObject *
 rpmte_Color(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("i", rpmteColor(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_PkgFileSize(rpmteObject * s)
-	/*@*/
 {
-    return Py_BuildValue("i", rpmtePkgFileSize(s->te));
+    return Py_BuildValue("L", rpmtePkgFileSize(s->te));
 }
 
-/*@null@*/
-static PyObject *
-rpmte_Depth(rpmteObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmteDepth(s->te));
-}
-
-/*@null@*/
-static PyObject *
-rpmte_Npreds(rpmteObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmteNpreds(s->te));
-}
-
-/*@null@*/
-static PyObject *
-rpmte_Degree(rpmteObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmteDegree(s->te));
-}
-
-/*@null@*/
 static PyObject *
 rpmte_Parent(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("i", rpmteParent(s->te));
 }
 
-/*@null@*/
-static PyObject *
-rpmte_Tree(rpmteObject * s)
-	/*@*/
+static PyObject * rpmte_Problems(rpmteObject * s)
 {
-    return Py_BuildValue("i", rpmteTree(s->te));
+    rpmps ps = rpmteProblems(s->te);
+    PyObject *problems = rpmps_AsList(ps);
+    rpmpsFree(ps);
+    return problems;
 }
 
-/*@null@*/
-static PyObject *
-rpmte_AddedKey(rpmteObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmteAddedKey(s->te));
-}
-
-/*@null@*/
+/*
 static PyObject *
 rpmte_DependsOnKey(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("i", rpmteDependsOnKey(s->te));
 }
+*/
 
-/*@null@*/
 static PyObject *
 rpmte_DBOffset(rpmteObject * s)
-	/*@*/
 {
     return Py_BuildValue("i", rpmteDBOffset(s->te));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_Key(rpmteObject * s)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies _Py_NoneStruct @*/
 {
     PyObject * Key;
 
@@ -227,79 +155,37 @@ rpmte_Key(rpmteObject * s)
     return Key;
 }
 
-/*@null@*/
 static PyObject *
 rpmte_DS(rpmteObject * s, PyObject * args, PyObject * kwds)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies _Py_NoneStruct @*/
 {
-    PyObject * TagN = NULL;
     rpmds ds;
-    rpmTag tag;
+    rpmTagVal tag;
     char * kwlist[] = {"tag", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:DS", kwlist, &TagN))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&:DS", kwlist,
+				tagNumFromPyObject, &tag))
 	return NULL;
 
-    tag = tagNumFromPyObject(TagN);
-    if (tag == -1) {
-	PyErr_SetString(PyExc_TypeError, "unknown tag type");
-	return NULL;
-    }
-
     ds = rpmteDS(s->te, tag);
     if (ds == NULL) {
-#ifdef	DYING
-	PyErr_SetString(PyExc_TypeError, "invalid ds tag");
-	return NULL;
-#else
-	Py_INCREF(Py_None);
-	return Py_None;
-#endif
+	Py_RETURN_NONE;
     }
-    return (PyObject *) rpmds_Wrap(rpmdsLink(ds, "rpmte_DS"));
+    return rpmds_Wrap(&rpmds_Type, rpmdsLink(ds));
 }
 
-/*@null@*/
 static PyObject *
 rpmte_FI(rpmteObject * s, PyObject * args, PyObject * kwds)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies _Py_NoneStruct @*/
 {
-    PyObject * TagN = NULL;
     rpmfi fi;
-    rpmTag tag;
-    char * kwlist[] = {"tag", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:FI", kwlist, &TagN))
-	return NULL;
-
-    tag = tagNumFromPyObject(TagN);
-    if (tag == -1) {
-	PyErr_SetString(PyExc_TypeError, "unknown tag type");
-	return NULL;
-    }
-
-    fi = rpmteFI(s->te, tag);
+    fi = rpmteFI(s->te);
     if (fi == NULL) {
-#ifdef	DYING
-	PyErr_SetString(PyExc_TypeError, "invalid fi tag");
-	return NULL;
-#else
-	Py_INCREF(Py_None);
-	return Py_None;
-#endif
+	Py_RETURN_NONE;
     }
-    return (PyObject *) rpmfi_Wrap(rpmfiLink(fi, "rpmte_FI"));
+    return rpmfi_Wrap(&rpmfi_Type, rpmfiLink(fi));
 }
 
-/** \ingroup py_c
- */
-/*@-fullinitblock@*/
-/*@unchecked@*/ /*@observer@*/
 static struct PyMethodDef rpmte_methods[] = {
-    {"Debug",	(PyCFunction)rpmte_Debug,	METH_VARARGS|METH_KEYWORDS,
-	NULL},
     {"Type",	(PyCFunction)rpmte_TEType,	METH_NOARGS,
 "te.Type() -> Type\n\
 - Return element type (rpm.TR_ADDED | rpm.TR_REMOVED).\n" },
@@ -323,25 +209,20 @@ static struct PyMethodDef rpmte_methods[] = {
 - Return element os.\n" },
     {"NEVR",	(PyCFunction)rpmte_NEVR,	METH_NOARGS,
 "te.NEVR() -> NEVR\n\
-- Return element name-version-release.\n" },
+- Return element name-[epoch:]version-release.\n" },
+    {"NEVRA",	(PyCFunction)rpmte_NEVRA,	METH_NOARGS,
+"te.NEVRA() -> NEVRA\n\
+- Return element name-[epoch:]version-release.arch\n" },
     {"Color",(PyCFunction)rpmte_Color,		METH_NOARGS,
 	NULL},
     {"PkgFileSize",(PyCFunction)rpmte_PkgFileSize,	METH_NOARGS,
 	NULL},
-    {"Depth",	(PyCFunction)rpmte_Depth,	METH_NOARGS,
-	NULL},
-    {"Npreds",	(PyCFunction)rpmte_Npreds,	METH_NOARGS,
-	NULL},
-    {"Degree",	(PyCFunction)rpmte_Degree,	METH_NOARGS,
-	NULL},
     {"Parent",	(PyCFunction)rpmte_Parent,	METH_NOARGS,
 	NULL},
-    {"Tree",	(PyCFunction)rpmte_Tree,	METH_NOARGS,
-	NULL},
-    {"AddedKey",(PyCFunction)rpmte_AddedKey,	METH_NOARGS,
-	NULL},
-    {"DependsOnKey",(PyCFunction)rpmte_DependsOnKey,	METH_NOARGS,
+    {"Problems",(PyCFunction)rpmte_Problems,	METH_NOARGS,
 	NULL},
+/*    {"DependsOnKey",(PyCFunction)rpmte_DependsOnKey,	METH_NOARGS,
+      NULL}, */
     {"DBOffset",(PyCFunction)rpmte_DBOffset,	METH_NOARGS,
 	NULL},
     {"Key",	(PyCFunction)rpmte_Key,		METH_NOARGS,
@@ -355,56 +236,19 @@ static struct PyMethodDef rpmte_methods[] = {
 - Return the TagN dependency set (or None). TagN must be 'Basenames'.\n" },
     {NULL,		NULL}		/* sentinel */
 };
-/*@=fullinitblock@*/
 
 /* ---------- */
 
-static int
-rpmte_print(rpmteObject * s, FILE * fp, /*@unused@*/ int flags)
-	/*@globals fileSystem @*/
-	/*@modifies fp, fileSystem @*/
-{
-    const char * tstr;
-    if (!(s && s->te))
-	return -1;
-    switch (rpmteType(s->te)) {
-    case TR_ADDED:	tstr = "++";	break;
-    case TR_REMOVED:	tstr = "--";	break;
-    default:		tstr = "??";	break;
-    }
-    fprintf(fp, "%s %s %s", tstr, rpmteNEVR(s->te), rpmteA(s->te));
-    return 0;
-}
-
-static PyObject * rpmte_getattro(PyObject * o, PyObject * n)
-	/*@*/
-{
-    return PyObject_GenericGetAttr(o, n);
-}
-
-static int rpmte_setattro(PyObject * o, PyObject * n, PyObject * v)
-	/*@*/
-{
-    return PyObject_GenericSetAttr(o, n, v);
-}
-
-/**
- */
-/*@unchecked@*/ /*@observer@*/
 static char rpmte_doc[] =
 "";
 
-/** \ingroup py_c
- */
-/*@-fullinitblock@*/
 PyTypeObject rpmte_Type = {
-	PyObject_HEAD_INIT(&PyType_Type)
-	0,				/* ob_size */
+	PyVarObject_HEAD_INIT(&PyType_Type, 0)
 	"rpm.te",			/* tp_name */
 	sizeof(rpmteObject),		/* tp_size */
 	0,				/* tp_itemsize */
 	(destructor)0,		 	/* tp_dealloc */
-	(printfunc) rpmte_print,	/* tp_print */
+	0,				/* tp_print */
 	(getattrfunc)0,		 	/* tp_getattr */
 	(setattrfunc)0,			/* tp_setattr */
 	0,				/* tp_compare */
@@ -415,12 +259,11 @@ PyTypeObject rpmte_Type = {
 	0,				/* tp_hash */
 	0,				/* tp_call */
 	0,				/* tp_str */
-	(getattrofunc) rpmte_getattro,	/* tp_getattro */
-	(setattrofunc) rpmte_setattro,	/* tp_setattro */
+	PyObject_GenericGetAttr,	/* tp_getattro */
+	PyObject_GenericSetAttr,	/* tp_setattro */
 	0,				/* tp_as_buffer */
-	Py_TPFLAGS_DEFAULT,		/* tp_flags */
+	Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,	/* tp_flags */
 	rpmte_doc,			/* tp_doc */
-#if Py_TPFLAGS_HAVE_ITER
 	0,				/* tp_traverse */
 	0,				/* tp_clear */
 	0,				/* tp_richcompare */
@@ -440,15 +283,13 @@ PyTypeObject rpmte_Type = {
 	0,				/* tp_new */
 	0,				/* tp_free */
 	0,				/* tp_is_gc */
-#endif
 };
-/*@=fullinitblock@*/
 
-rpmteObject * rpmte_Wrap(rpmte te)
+PyObject * rpmte_Wrap(PyTypeObject *subtype, rpmte te)
 {
-    rpmteObject *s = PyObject_New(rpmteObject, &rpmte_Type);
-    if (s == NULL)
-	return NULL;
+    rpmteObject *s = (rpmteObject *)subtype->tp_alloc(subtype, 0);
+    if (s == NULL) return NULL;
+
     s->te = te;
-    return s;
+    return (PyObject *) s;
 }
