@@ -1,20 +1,17 @@
 #include "system.h"
 const char *__progname;
 
-#include <rpmbuild.h>
-#include <argv.h>
-#include <rpmds.h>
-#include <rpmfc.h>
+#include <rpm/rpmbuild.h>
+#include <rpm/argv.h>
+#include <rpm/rpmds.h>
+#include <rpm/rpmfc.h>
 
 #include "debug.h"
 
-/*@unchecked@*/
 char *progname;
 
-/*@unchecked@*/
 static int print_provides;
 
-/*@unchecked@*/
 static int print_requires;
 
 static void rpmdsPrint(const char * msg, rpmds ds, FILE * fp)
@@ -46,7 +43,7 @@ static struct poptOption optionsTable[] = {
 };
 
 int
-main(int argc, char *const argv[])
+main(int argc, char *argv[])
 {
     poptContext optCon;
     ARGV_t av = NULL;
@@ -56,18 +53,16 @@ main(int argc, char *const argv[])
     int xx;
 char buf[BUFSIZ];
 
-/*@-modobserver@*/
     if ((progname = strrchr(argv[0], '/')) != NULL)
 	progname++;
     else
 	progname = argv[0];
-/*@=modobserver@*/
 
     optCon = rpmcliInit(argc, argv, optionsTable);
     if (optCon == NULL)
 	goto exit;
 
-    av = poptGetArgs(optCon);
+    av = (ARGV_t) poptGetArgs(optCon);
     ac = argvCount(av);
 
     if (ac == 0) {
@@ -87,21 +82,20 @@ char buf[BUFSIZ];
     xx = argvSort(av, NULL);
 
     /* Build file class dictionary. */
-    fc = rpmfcNew();
+    fc = rpmfcCreate(getenv("RPM_BUILD_ROOT"), 0);
     xx = rpmfcClassify(fc, av, NULL);
 
     /* Build file/package dependency dictionary. */
     xx = rpmfcApply(fc);
 
 if (_rpmfc_debug) {
-sprintf(buf, "final: files %d cdict[%d] %d%% ddictx[%d]", fc->nfiles, argvCount(fc->cdict), ((100 * fc->fknown)/fc->nfiles), argiCount(fc->ddictx));
 rpmfcPrint(buf, fc, NULL);
 }
 
     if (print_provides)
-	rpmdsPrint(NULL, fc->provides, stdout);
+	rpmdsPrint(NULL, rpmfcProvides(fc), stdout);
     if (print_requires)
-	rpmdsPrint(NULL, fc->requires, stdout);
+	rpmdsPrint(NULL, rpmfcRequires(fc), stdout);
 
     fc = rpmfcFree(fc);
 
